@@ -46,9 +46,19 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
         self.liveutils._iscsi_initiator = mock.MagicMock()
         self.liveutils._jobutils = mock.Mock()
 
-        self.liveutils._get_conn_v2 = mock.MagicMock(return_value=self._conn)
+        self.liveutils._get_wmi_obj = mock.MagicMock(return_value=self._conn)
         self.liveutils._conn_v2 = self._conn
 
+    def test_get_conn_v2(self):
+        self.liveutils._get_wmi_obj.side_effect = exceptions.x_wmi(
+            com_error=mock.Mock())
+
+        self.assertRaises(exceptions.HyperVException,
+                          self.liveutils._get_conn_v2, '.')
+
+        self.liveutils._get_wmi_obj.assert_called_once_with(
+            self.liveutils._wmi_namespace % '.', compatibility_mode=True)
+
     def test_check_live_migration_config(self):
         mock_migr_svc = (
             self._conn.Msvm_VirtualSystemMigrationService.return_value[0])
@@ -90,56 +100,38 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
                           mock_conn_v2, self._FAKE_VM_NAME)
 
     def test_destroy_planned_vm(self):
-        mock_conn_v2 = mock.MagicMock()
         mock_planned_vm = mock.MagicMock()
-        mock_vs_man_svc = mock.MagicMock()
-        mock_conn_v2.Msvm_VirtualSystemManagementService.return_value = [
-            mock_vs_man_svc]
         mock_planned_vm.path_.return_value = mock.sentinel.planned_vm_path
+        mock_vs_man_svc = self.liveutils._vs_man_svc
         mock_vs_man_svc.DestroySystem.return_value = (
             mock.sentinel.job_path, mock.sentinel.ret_val)
 
-        self.liveutils._destroy_planned_vm(mock_conn_v2, mock_planned_vm)
+        self.liveutils._destroy_planned_vm(mock_planned_vm)
 
-        mock_msvms_cls = mock_conn_v2.Msvm_VirtualSystemManagementService
-        mock_msvms_cls.assert_called_once_with()
         mock_vs_man_svc.DestroySystem.assert_called_once_with(
             mock.sentinel.planned_vm_path)
         self.liveutils._jobutils.check_ret_val.assert_called_once_with(
             mock.sentinel.ret_val,
             mock.sentinel.job_path)
 
-    def test_get_planned_vms(self):
-        mock_conn_v2 = mock.MagicMock()
-        mock_vm = self._get_vm()
-        mock_conn_v2.Msvm_PlannedComputerSystem.return_value = (
-            mock.sentinel.planned_vms)
-
-        planned_vms = self.liveutils._get_planned_vms(mock_conn_v2, mock_vm)
-
-        mock_conn_v2.Msvm_PlannedComputerSystem.assert_called_once_with(
-            Name=self._FAKE_VM_NAME)
-        self.assertEqual(mock.sentinel.planned_vms, planned_vms)
-
+    @ddt.data({'planned_vm': None}, {'planned_vm': mock.sentinel.planned_vm})
+    @ddt.unpack
     @mock.patch.object(livemigrationutils.LiveMigrationUtils,
                        '_destroy_planned_vm')
     @mock.patch.object(livemigrationutils.LiveMigrationUtils,
-                       '_get_planned_vms')
-    def test_destroy_existing_planned_vms(self, mock_get_planned_vms,
-                                          mock_destroy_planned_vm):
-        mock_conn_v2 = mock.sentinel.conn_v2
-        mock_planned_vms = [mock.sentinel.planned_vm,
-                            mock.sentinel.another_planned_vm]
-        mock_get_planned_vms.return_value = mock_planned_vms
+                       '_get_planned_vm')
+    def test_destroy_existing_planned_vm(self, mock_get_planned_vm,
+                                         mock_destroy_planned_vm, planned_vm):
+        mock_get_planned_vm.return_value = planned_vm
 
-        self.liveutils._destroy_existing_planned_vms(mock_conn_v2,
-                                                     mock.sentinel.vm)
+        self.liveutils.destroy_existing_planned_vm(mock.sentinel.vm_name)
 
-        mock_get_planned_vms.assert_called_once_with(mock_conn_v2,
-                                                     mock.sentinel.vm)
-        mock_destroy_planned_vm.assert_has_calls(
-            [mock.call(mock_conn_v2, mock.sentinel.planned_vm),
-             mock.call(mock_conn_v2, mock.sentinel.another_planned_vm)])
+        mock_get_planned_vm.assert_called_once_with(
+            mock.sentinel.vm_name, self._conn)
+        if planned_vm:
+            mock_destroy_planned_vm.assert_called_once_with(planned_vm)
+        else:
+            self.assertFalse(mock_destroy_planned_vm.called)
 
     def test_create_planned_vm_helper(self):
         mock_vm = mock.MagicMock()
@@ -310,19 +302,12 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
             MigrationSettingData=mock_vsmsd.GetText_.return_value,
             NewResourceSettingData=mock.sentinel.FAKE_RASD_PATH)
 
-    def test_live_migrate_multiple_planned_vms(self):
-        mock_vm = self._get_vm()
-        self._conn.Msvm_PlannedComputerSystem.return_value = [
-            mock_vm, mock_vm]
-
-        self.assertRaises(exceptions.OSWinException,
-                          self.liveutils.live_migrate_vm,
-                          mock.sentinel.vm_name,
-                          mock.sentinel.host)
-
     @ddt.data(True, False)
+    @mock.patch.object(livemigrationutils.LiveMigrationUtils,
+                       '_get_planned_vm')
     @mock.patch.object(livemigrationutils, 'vmutils')
-    def test_live_migrate_no_planned_vm(self, migrate_disks, mock_vm_utils):
+    def test_live_migrate_no_planned_vm(self, migrate_disks, mock_vm_utils,
+                                        mock_get_planned_vm):
         mock_vm_utils_remote = mock_vm_utils.VMUtils.return_value
         mock_vm = self._get_vm()
 
@@ -340,7 +325,7 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
                 _get_vhd_setting_data=mock.DEFAULT,
                 _live_migrate_vm=mock.DEFAULT):
 
-            self._conn.Msvm_PlannedComputerSystem.return_value = []
+            mock_get_planned_vm.return_value = None
             disk_paths = {
                 mock.sentinel.FAKE_IDE_PATH: mock.sentinel.FAKE_SASD_RESOURCE}
             self.liveutils._get_physical_disk_paths.return_value = disk_paths
@@ -352,7 +337,8 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
             self.liveutils.live_migrate_vm(mock.sentinel.vm_name,
                                            mock.sentinel.FAKE_HOST,
                                            migrate_disks=migrate_disks)
-
+            mock_get_planned_vm.assert_called_once_with(
+                mock.sentinel.vm_name, self._conn)
             self.liveutils._get_remote_disk_data.assert_called_once_with(
                 mock_vm_utils_remote, disk_paths, mock.sentinel.FAKE_HOST)
             self.liveutils._create_planned_vm.assert_called_once_with(
@@ -379,7 +365,9 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
                 mock.sentinel.FAKE_HOST,
                 expected_migr_type)
 
-    def test_live_migrate_single_planned_vm(self):
+    @mock.patch.object(
+        livemigrationutils.LiveMigrationUtils, '_get_planned_vm')
+    def test_live_migrate_single_planned_vm(self, mock_get_planned_vm):
         mock_vm = self._get_vm()
 
         mock_migr_svc = self._conn.Msvm_VirtualSystemMigrationService()[0]
@@ -392,7 +380,7 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
                 _get_vhd_setting_data=mock.DEFAULT,
                 _live_migrate_vm=mock.DEFAULT):
 
-            self._conn.Msvm_PlannedComputerSystem.return_value = [mock_vm]
+            mock_get_planned_vm.return_value = mock_vm
             self.liveutils.live_migrate_vm(mock.sentinel.vm_name,
                                            mock.sentinel.FAKE_HOST)
             self.liveutils._live_migrate_vm.assert_called_once_with(
@@ -401,6 +389,8 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
                 self.liveutils._get_vhd_setting_data.return_value,
                 mock.sentinel.FAKE_HOST,
                 self.liveutils._MIGRATION_TYPE_VIRTUAL_SYSTEM_AND_STORAGE)
+            mock_get_planned_vm.assert_called_once_with(
+                mock.sentinel.vm_name, self._conn)
 
     @mock.patch.object(vmutils, 'VMUtils')
     @mock.patch.object(livemigrationutils.LiveMigrationUtils, '_get_vm')
@@ -411,7 +401,7 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
     @mock.patch.object(livemigrationutils.LiveMigrationUtils,
                        '_create_planned_vm')
     @mock.patch.object(livemigrationutils.LiveMigrationUtils,
-                       '_destroy_existing_planned_vms')
+                       'destroy_existing_planned_vm')
     @mock.patch.object(livemigrationutils.LiveMigrationUtils,
                        '_get_disk_data')
     def test_create_planned_vm(self, mock_get_disk_data,
@@ -424,7 +414,7 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
         mock_vm = mock.MagicMock()
         mock_get_vm.return_value = mock_vm
         mock_conn_v2 = mock.MagicMock()
-        self.liveutils._get_conn_v2.return_value = mock_conn_v2
+        self.liveutils._get_wmi_obj.return_value = mock_conn_v2
 
         mock_get_disk_data.return_value = mock.sentinel.disk_data
         mock_get_ip_address_list.return_value = mock.sentinel.ip_address_list
@@ -439,8 +429,8 @@ class LiveMigrationUtilsTestCase(test_base.OsWinBaseTestCase):
                                          mock.sentinel.host,
                                          mock.sentinel.disk_path_mapping)
 
-        mock_destroy_existing_planned_vm.assert_called_once_with(self._conn,
-                                                                 mock_vm)
+        mock_destroy_existing_planned_vm.assert_called_once_with(
+            mock.sentinel.vm_name)
         mock_get_ip_address_list.assert_called_once_with(self._conn, dest_host)
         mock_get_disk_data.assert_called_once_with(
             mock.sentinel.vm_name,
