@@ -112,7 +112,6 @@ class VMUtils(baseutils.BaseUtilsVirt):
 
     _vm_power_states_map = {constants.HYPERV_VM_STATE_ENABLED: 2,
                             constants.HYPERV_VM_STATE_DISABLED: 3,
-                            constants.HYPERV_VM_STATE_SHUTTING_DOWN: 4,
                             constants.HYPERV_VM_STATE_REBOOT: 11,
                             constants.HYPERV_VM_STATE_PAUSED: 9,
                             constants.HYPERV_VM_STATE_SUSPENDED: 6}
@@ -277,7 +276,7 @@ class VMUtils(baseutils.BaseUtilsVirt):
         virtual_system_type = self._get_virtual_system_type(is_planned_vm)
 
         vmsetting = self._lookup_vm_check(
-            vm_name, virtual_system_type=virtual_system_type)
+            vm_name, for_update=True, virtual_system_type=virtual_system_type)
 
         if configuration_root_dir:
             vmsetting.ConfigurationDataRoot = configuration_root_dir
